// Mac Pierce 2021
// Code to run the project 'A Scanner Darkly'
// www.macpierce.com/a-scanner-darkly

// 2 of 2 code blocks, RX - Receive
// Made for the WEMOS D1 Mini Pro ESP8266 Modules

/* **** Update Log *******
 *  - 210410 - v0-01
 *    - Building it up, settled on painless mesh for mesh network.
 *    - Split TX and RX Code.
 *    - Started building in dip switch addressing.
 *  - 210508 - v0-2
 *    - Building the message receiving & dip
 *    - Message Receiving built.
 *  - 210707 - v2-0
 *    - Attempting to incorporate ESP-Now for wifi
 *  - 210710 - v2-01
 *    - Appears that task scheduler and ESP-Now are incompatible. Removing Task Scheduler.
 *    - Task Scheduler removed
 *  - 210714 - v3-00
 *    - Removed Painless Mesh
 *    - Incoporated ESP-Now
 *    - DONE !
*/

#include <ESP8266WiFi.h>
#include <espnow.h>

#define LEDPIN1 D8
#define SPIN1 D7
#define SPIN2 D6
#define SPIN3 D5
#define SPIN4 D0

// ----------- Variable Def ------

uint8_t dID; // The device ID
String charPattern; // Holds the currently pushed pattern
bool eLED = false; // Whether or not the light is on;

// --- Timing ---
const int lcPush = 100; // The number of milliseconds it waits to push char sequence to the light
long pMil;
long cMil;


// --- Debug ---
bool debug = true;

// --- Callback function when data is recveived from conductor (https://stackoverflow.com/questions/64998264/cannot-send-string-properly-with-esp-now)
void OnDataRecv(uint8_t *mac_addr, uint8_t *data, uint8_t len){
  char* buff = (char*) data;
  charPattern = String(buff);
  
  if(Serial){
    Serial.print("Received - " + charPattern + " | ID is " + dID);
    if(charPattern.charAt(dID) == '1'){
      Serial.println(" | ENABLED");
    }
    else{
      Serial.println(" | DISABLED");
    }
  }
  ID();
}

// --------------------- Project Setup ---------------------------------------------------------------
void setup(){
  Serial.begin(115200);

  pinMode(LEDPIN1,OUTPUT); // The pin that controlls the LED
  pinMode(SPIN1,OUTPUT); // The pins that read the dip switches
  pinMode(SPIN2,OUTPUT);
  pinMode(SPIN3,OUTPUT);
  pinMode(SPIN4,OUTPUT);

  // ------------ ESP-Now Stuff------------
  WiFi.mode(WIFI_STA);
  if (esp_now_init() != 0) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }

  esp_now_set_self_role(ESP_NOW_ROLE_SLAVE);
  esp_now_register_recv_cb(OnDataRecv);

  ID(); // Sets the ID when the node is powered
  charPattern = "000000000000000"; // Sets the string to all negative to start
  
  if(Serial){
   Serial.println();
   Serial.println();
   Serial.println("A SCANNER DARKLY");
   Serial.println("Mac Pierce - 2021");
   Serial.println();
   Serial.println("*** Starting node. ID is " + String(dID) + " ***"); 
  }
}

// ------------------------- Main Loop ------------------------------------------------------------

void loop() {
  cMil = millis(); // This block checks to see if it's been more than 100ms since it last received a message.
  if(cMil >= (pMil+lcPush)){
    lightCheck();
    pMil = cMil;
  }
}

// ------------------------- Project Functions -----------------------------------------------------

void lightCheck(){ // --------------- Reads the most recent broadcast message, and controlls the light
  if(charPattern.charAt(dID) == '1'){
    eLED = true;
    digitalWrite(LEDPIN1, HIGH);
  }
  else{
    eLED = false;
    digitalWrite(LEDPIN1, LOW);
  }
}

void ID(){ // Function that checks the dip switches and assigns the node's ID, which can be 0-15
  dID = 0;
  
  digitalWrite(SPIN1, HIGH);
  digitalWrite(SPIN2, HIGH);
  digitalWrite(SPIN3, HIGH);
  digitalWrite(SPIN4, HIGH);

  if(!digitalRead(SPIN1)){
    dID +=8;}
  if(!digitalRead(SPIN2)){
    dID +=4;}
  if(!digitalRead(SPIN3)){
    dID +=2;}
  if(!digitalRead(SPIN4)){
    dID +=1;}
}
